/***************************************************************************
 * sound_manager.h  -  header for the corresponding cpp file
 *
 * Copyright (C) 2006 - 2008 Florian Richter
 ***************************************************************************/
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
   
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __SOUND_MANAGER_H__
#define __SOUND_MANAGER_H__

#include "../audio/audio.h"
#include "../core/obj_manager.h"

/* *** *** *** *** *** *** cSound_Manager *** *** *** *** *** *** *** *** *** *** *** */

/*  Keeps track of all sounds in memory
 *
 * Operators:
 * - cSound_Manager [path]
 * - cSound_Manager [ID]
*/
class cSound_Manager : public cObject_Manager<cAudio_Sound>
{
public:
	cSound_Manager( void );
	virtual ~cSound_Manager( void );

	// Return the Sound from Path
	virtual cAudio_Sound *Get_Pointer( string path );

	// Return the Sound Path as string
	string Get_Path( unsigned int identifier );

	/* Add a Sound
	 * Should always have the path set
	 */
	void Add( cAudio_Sound *item );

	cAudio_Sound *operator [] ( unsigned int identifier )
	{
		return cObject_Manager<cAudio_Sound>::Get_Pointer( identifier );
	}
	
	cAudio_Sound *operator [] ( string path )
	{
		return Get_Pointer( path );
	}

	// Delete all Sounds, but keep object vector entries
	void Delete_Sounds( void );

private:
	// sounds loaded since initialization
	unsigned int load_count;
};

/* *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** */

// Sound Manager
extern cSound_Manager *pSound_Manager;

/* *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** */

#endif
